@@ -3,7 +3,7 @@
  * Created : 26 Apr 2017 by Evgeny Khramtsov <ekhramtsov@process-one.net>
  *
  *
- * Copyright (C) 2002-2019 ProcessOne, SARL. All Rights Reserved.
+ * Copyright (C) 2002-2020 ProcessOne, SARL. All Rights Reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -26,21 +26,34 @@
 
 #define MAX_UINT64 ((ErlNifUInt64) -1)
 
-static unsigned int max_counters = 0;
-static unsigned int counters_num = 0;
-static ErlNifUInt64 *counters = NULL;
-static ErlNifMutex *counters_lock = NULL;
+typedef struct State {
+  unsigned int max_counters;
+  unsigned int counters_num;
+  ErlNifUInt64 *counters;
+  ErlNifMutex *counters_lock;
+} State;
 
 static int load(ErlNifEnv* env, void** priv, ERL_NIF_TERM max)
 {
   size_t size;
+  unsigned int max_counters;
+
+  State *state;
+
+  if (!(state = enif_alloc(sizeof(*state)))) {
+      return ENOMEM;
+  }
+
   if (enif_get_uint(env, max, &max_counters)) {
     size = sizeof(ErlNifUInt64) * max_counters;
-    if ((counters = enif_alloc(size))) {
-      memset(counters, -1, size);
-      counters_num = 0;
-      counters_lock = enif_mutex_create("counters_lock");
+    if ((state->counters = enif_alloc(size))) {
+      memset(state->counters, -1, size);
+      state->counters_num = 0;
+      state->counters_lock = enif_mutex_create("counters_lock");
+      *priv = state;
       return 0;
+    } else {
+      enif_free(state);
     }
   }
 
@@ -49,34 +62,54 @@ static int load(ErlNifEnv* env, void** priv, ERL_NIF_TERM max)
 
 static void unload(ErlNifEnv* env, void* priv)
 {
-  if (counters_lock) enif_mutex_destroy(counters_lock);
-  if (counters) enif_free(counters);
+  if (!priv)
+    return;
+
+  State *state = (State*)priv;
+
+  if (state->counters_lock) enif_mutex_destroy(state->counters_lock);
+  if (state->counters) enif_free(state->counters);
+
+  enif_free(state);
+}
+
+static int upgrade(ErlNifEnv* caller_env, void** priv_data, void** old_priv_data, ERL_NIF_TERM load_info) {
+  if (!*old_priv_data) {
+    return 1;
+  }
+
+  *priv_data = *old_priv_data;
+  *old_priv_data = NULL;
+
+  return 0;
 }
 
 static ERL_NIF_TERM new_counter(ErlNifEnv* env, int argc,
 				const ERL_NIF_TERM argv[])
 {
   unsigned int counter;
-  enif_mutex_lock(counters_lock);
-  if (counters_num < max_counters) {
-    counter = counters_num++;
-    counters[counter] = 0;
-    enif_mutex_unlock(counters_lock);
+  State *state = (State*)enif_priv_data(env);
+  enif_mutex_lock(state->counters_lock);
+
+  if (state->counters_num < state->max_counters) {
+    counter = state->counters_num++;
+    state->counters[counter] = 0;
+    enif_mutex_unlock(state->counters_lock);
     return enif_make_tuple2(env, enif_make_atom(env, "ok"),
 			    enif_make_ulong(env, counter));
   } else {
-    for (counter = 0; counter < max_counters; counter++) {
-      if (counters[counter] == MAX_UINT64) {
-	counters[counter] = 0;
-	enif_mutex_unlock(counters_lock);
-	return enif_make_tuple2(env, enif_make_atom(env, "ok"),
+    for (counter = 0; counter < state->max_counters; counter++) {
+      if (state->counters[counter] == MAX_UINT64) {
+        state->counters[counter] = 0;
+        enif_mutex_unlock(state->counters_lock);
+        return enif_make_tuple2(env, enif_make_atom(env, "ok"),
 				enif_make_ulong(env, counter));
       }
     }
-    enif_mutex_unlock(counters_lock);
+    enif_mutex_unlock(state->counters_lock);
     return enif_make_tuple2(env, enif_make_atom(env, "error"),
-			    enif_make_tuple2(env, enif_make_atom(env, "system_limit"),
-					     enif_make_uint(env, max_counters)));
+                            enif_make_tuple2(env, enif_make_atom(env, "system_limit"),
+                                             enif_make_uint(env, state->max_counters)));
   }
 }
 
@@ -84,9 +117,11 @@ static ERL_NIF_TERM delete_counter(ErlNifEnv* env, int argc,
 				   const ERL_NIF_TERM argv[])
 {
   unsigned int counter;
+  State *state = (State*)enif_priv_data(env);
+
   if (enif_get_uint(env, argv[0], &counter))
-    if (counter < max_counters) {
-      counters[counter] = MAX_UINT64;
+    if (counter < state->max_counters) {
+      state->counters[counter] = MAX_UINT64;
       return enif_make_atom(env, "ok");
     }
 
@@ -97,10 +132,12 @@ static ERL_NIF_TERM incr_counter(ErlNifEnv* env, int argc,
 				 const ERL_NIF_TERM argv[])
 {
   unsigned int counter;
+  State *state = (State*)enif_priv_data(env);
+
   if (enif_get_uint(env, argv[0], &counter))
-    if (counter < max_counters) {
-      if (counters[counter] != MAX_UINT64)
-	return enif_make_uint64(env, __sync_add_and_fetch(counters + counter, 1));
+    if (counter < state->max_counters) {
+      if (state->counters[counter] != MAX_UINT64)
+        return enif_make_uint64(env, __sync_add_and_fetch(state->counters + counter, 1));
     }
 
   return enif_make_badarg(env);
@@ -111,11 +148,13 @@ static ERL_NIF_TERM get_counter(ErlNifEnv* env, int argc,
 {
   unsigned int counter;
   ErlNifUInt64 val;
+  State *state = (State*)enif_priv_data(env);
+
   if (enif_get_uint(env, argv[0], &counter))
-    if (counter < max_counters) {
-      val = counters[counter];
+    if (counter < state->max_counters) {
+      val = state->counters[counter];
       if (val != MAX_UINT64)
-	return enif_make_uint64(env, val);
+        return enif_make_uint64(env, val);
     }
 
   return enif_make_badarg(env);
@@ -129,4 +168,4 @@ static ErlNifFunc nif_funcs[] =
       {"get_counter_nif", 1, get_counter}
     };
 
-ERL_NIF_INIT(ets_cache, nif_funcs, load, NULL, NULL, unload)
+ERL_NIF_INIT(ets_cache, nif_funcs, load, NULL, upgrade, unload)
